% This code runs the sublinearized CoSaMP for the exactly sparse case
% without noise.
%
% based on SHT implementation 2018 by Bosu Choi
% version with speed improvements and generalizations 2018-2019 by Toni Volkmer

N = 200; % expansion of I_{N,d}
s = 25; % sparsity
d = 10; % spatial dimension
m_ls = 50*s; % number of samples m_CE

m_1 = 8*s;
% m_1 = d*round(2^(d/2)*(2*s)^1.5); %sample number m_1 for pairing (Chebyshev/Legendre)

m_2 = s;
% m_2 = round(2^(d/2)*s); %sample number m_2 for pairing (Chebyshev/Legendre)

m_entryid = [m_1,m_2]; %entryID via compressed sensing approach (SHT II paper)
L = 0;

% m_entryid = 10*s; %(Use this and L>0 for SHT I paper entryID)
% L=10; % number of proxy functions h_{j,k}, only used in case of scalar m_entryid

% m_entryid = []; % do not perform entryID

basis_types = cell(d,1);
for jj=1:d
  basis_types{jj} = 'fourier';
%   basis_types{jj} = 'legendre_preconditioned';
end

tau=10^(-6); % threshold used in entryId, using tau instead of 0 due to the machine precision
cosamp_max_iter = 10; % maximum number of CoSaMP iterations

fprintf('N = %d, d = %d, s = %d, possible basis indices N^d = %8g\n', N, d, s, N^d);

[iter,error_n,error_c,aa,cc,tElapsed_sam,tElapsed_tot,SF,sam_num]=sublin_cosamp_v2_func_noise(N,d,basis_types,s,m_entryid,L,tau,m_ls,m_1,m_2,cosamp_max_iter,inf,'pairing');

if SF
  disp('detection successful, all basis coefficients correctly detected');
  fprintf('CoSaMP required %d iterations, %d samples and %.3f seconds runtime\n', iter, sam_num, tElapsed_tot);
else
  disp('detection failed, at least one basis coefficient not detected');
end
